/*

Wink Robot
Wink_PreLoadedDemo_Rev01
Version 1.0 12/2015

This sketch is pre-loaded onto Wink robots before
shipping. You can use this source sketch to re-load
these behaviors at any time.

This code was written by Kevin King and
Dustin Soodak for Plum Geek LLC.
Portions from other open source projects where noted.
This code is licensed under:
Creative Commons Attribution-ShareAlike 2.0 Generic (CC BY-SA 2.0)
https://creativecommons.org/licenses/by-sa/2.0/
Visit http://www.plumgeek.com for Wink information.
Visit http://www.arduino.cc to learn about the Arduino.

*/

#include "WinkHardware.h"


// Variables used in Barrier Detect Behavior
int motorSpeedLeft = 100;           //adjust left/right motor speeds to make Wink drive straight
int motorSpeedRight = 100;          //adjust left/right motor speeds to make Wink drive straight
int barrierThreshold=10;            //controls how sensitive Wink is to barrier. Make smaller to make Wink
                                    //see the barrier further away (but may false trigger), make larger
                                    //to make Wink get closer to a barrier before stopping. 10 is a good start.
int leftReflection;                 //
int centerReflection;               //these variables are used by the barrierDetect() function
int rightReflection;                // 

// Variables used in LightSeek Behavior
int rightMotorSpeedBase = 40; //Base motor speed. Increase these numbers to make Wingo drive faster when seeking. A starting value of about 20 is good.
int leftMotorSpeedBase = 40;  //adjust these two numbers to make Wink seek straight, or make them different to make him seek in an arc or circle.




void setup(){
  hardwareBegin();        //initialize Wink's brain to work with his circuitry
  playStartChirp();       //Play startup chirp and blink eyes
}


void loop(){  
  
  winkDanceBehavior();
  barrierDetectBehavior();    //run the Barrier Detect behavior
  lightSeekBehavior();        //run the Light Seeking behavior
  winkIRBehavior();        //run the IR remote demo

} //end of loop()   //closing curly of the “loop()”


////////////////////////////////////////////
/// Wink Dance Behavior Code      //////////
////////////////////////////////////////////

void winkDanceBehavior(void){

  //Setup for this example
  int r,g,b;                              //declare variables used in this example
  int right,left;
  int i;

//  eyesRGB(0,255,0);                     //un-comment this section to make Wink wait for
//  delay(500);                           //a button press from the IR remote control before
//  while(digitalRead(ExtIO)==1){         //starting to dance. This is useful to syncrhonize
//    //do nothing                        //several Winks to dance together.
//    }

  delay(461);                             //461 milliseconds is 1 beat at 130 bpm
  while(1){

    r = random(120);                      // random pick Red eye component
    g = random(120);                      // random pick Green eye component
    b = random(120);                      // random pick Blue eye component
    right = random(-200,200);             // pick number between -200 and 200 for right motor speed
    left = random(-200,200);              // pick number between -200 and 200 for left motor speed

    eyesRGB(r,g,b);                       // set eye color
    motors(left,right);                   // set motors to the random speeds and direction
    delay(115);                           // play first note for 100 mS
    
    r = random(120);                      // random pick Red eye component
    g = random(120);                      // random pick Green eye component
    b = random(120);                      // random pick Blue eye component
    eyesRGB(r,g,b);                       // set new eye color
    delay(115);                           // total delay of 461ms matches tempo at 130 bpm

    r = random(120);                      // random pick Red eye component
    g = random(120);                      // random pick Green eye component
    b = random(120);                      // random pick Blue eye component
    eyesRGB(r,g,b);                       // set new eye color
    delay(230);

    if(buttonPressed()){                  //Exit this behavior if button is pressed
      motors(0,0);                        //make sure motors are stopped
      for(i=0;i<2;i++){
        eyesRGB(0,120,0);
        beep(20);
        delay(10);
        eyesRGB(0,0,0);
        delay(100);
      }
      delay(300);
      break;  //break causes program to exit the while(1) loop
    }
     
  }//end of while(1);
}
////////////////////////////////////////////
/// End Of Wink Dance Behavior Code      ///
////////////////////////////////////////////


////////////////////////////////////////////
/// Barrier Detect Behavior Code  //////////
////////////////////////////////////////////
void barrierDetectBehavior(void){

  int i;
  int noBarrierLevel; // declaire variable noBarrierLevel. This holds the centerReflection value
                      // that is seen when no barrier is present. (This is light that is reflected
                      // from the running surface itself rather than a barrier). When this behavior
                      // begins, place Wink on the intended running surface at least 4 inches away
                      // from any barrier and move your hands away until he starts moving. Wink will
                      // sample this value just before he starts moving.
 
  delay(500);
  eyesRGB(100,0,100);
  beep(30);
  eyesOff();
  delay(1500);
  noBarrierLevel = barrierDetect();    //sample what "no barrier" looks like
  
  while(1){     //stay in this loop forever until button is pressed
    motors(motorSpeedLeft,motorSpeedRight);
    eyesOff();

    if (barrierDetect()-barrierThreshold > noBarrierLevel){    //if a barrier is seen...
      motors(0,0);                                //stop motors
      eyesRGB(120,10,120);                        //this sets color of eyes when a barrier is seen
      delay(100);                             
      if (rightReflection-1 > leftReflection){    //determine if barrier was on the left or right of Wink
        motors(-230,230);                         //This is how "hard" Wink turns LEFT when barrier is seen
      }
      else {
        motors(230,-230);                         //This is how "hard" Wink turns RIGHT when barrier is seen
      }
      delay(40);                                  //This controls how long the turn will last. 40 is good start.
                                                  //Larger numbers here make Wink rotate further
      eyesOff();
      delay(20);                                  //Short delay before motors stop spinning
      motors(0,0);
    }
    
    if(buttonPressed()){                          //Exit this behavior if button is pressed
      motors(0,0);                                //make sure motors are stopped
      for(i=0;i<2;i++){
        eyesRGB(0,120,0);
        beep(20);
        delay(10);
        eyesRGB(0,0,0);
        delay(100);
      }
      delay(300);
      break;        //break causes program to exit the while(1) loop
    }
  } // end of while(1);
}
////////////////////////////////////////////
/// End Of Barrier Detect Behavior Code  ///
////////////////////////////////////////////


////////////////////////////////////////////
/// Light Seeking Behavior Code   //////////
////////////////////////////////////////////
void lightSeekBehavior(void){
  //Setup for this example
                            
  int leftMotorSpeed, rightMotorSpeed;                  //Base motor speed. Increase these numbers to make Ringo drive faster. A starting value of about 120 is good.                           
  int turnStrengthBase = 50;                            //this determines how "hard" ringo will turn toward the light. Larger numbers make for sharper turns
  int turnStrengthBoost=0;
  int turnStrength;
  int deadband = 1;                                     //if the two sensors are reading  within this "dead band" then Ringo will just go straight
  int leftSensor, rightSensor, sensorDiff, centerSensor;
  int speedBoost;
  int leftSensorOffset = 0;                             //If you observe that one of the light sensors appears to be a bit more attracted to light than
  int rightsensorOffset = 0;                            //the other sensor (which is normal as there is some minor varience in the sensors and the
                                                        //amplification circuit for each side), then you can make one sensor a bit less sensitive by
                                                        //changing "leftSensorOffset" or "rightsensorOffset" for the applicable side to a negative number. 
                                                        //Normally you don't need much to accomplish this. A value of -1 to -5 is usually enough.



  int leftReflection, rightReflection, centerReflection;  //these values are updated by calling barrierDetect(). The values are the reflection measured
                                                          //from each of the three sensors reflected from the barrier.
  int i;

  
  motors(leftMotorSpeedBase+40,rightMotorSpeedBase+40);
  delay(1000);                 //drive "straight" for 1 second to observe motor drift
                               //if Ringo tends to veer to the Right, increase "rightMotorSpeedBase" above a bit (maybe 10 at a time)
                               //if Ringo tends to veer to the Left, increase "leftMotorSpeedBase" above a bit (maybe 10 at a time)
  while(1){

     eyesOff();
     delayMicroseconds(200);                                         //let sensors settle after eyes are turned off
     leftSensor = analogRead(AmbientSenseLeft)+leftSensorOffset;     // get some initial values for the light sensors
     rightSensor = analogRead(AmbientSenseRight)+rightsensorOffset;  // get some initial values for the light sensors
     centerSensor = analogRead(AmbientSenseCenter);
     sensorDiff = abs(leftSensor-rightSensor);                // get the difference between the two sensors (absolute value)
     
     speedBoost = centerSensor / 2;                           // base speed boost on centerSensor
     speedBoost = constrain(speedBoost,0,250);                // make sure speed boost doesn't exceed a limit. The 3rd argument sets max eye brightness in bright light.
     leftMotorSpeed = leftMotorSpeedBase + speedBoost;
     rightMotorSpeed = rightMotorSpeedBase + speedBoost;
     turnStrengthBoost = centerSensor/16;                     // turn strength increases as light level rises
     turnStrength = turnStrengthBase + turnStrengthBoost;

     if (speedBoost < 10){
        eyesOff();    //turn off eyes when light level is below a certain threshold
     }
     else {
        eyesRGB(0,0,speedBoost);         // go glow eyes when light is observed and tracked
     }                                   // eyes get brighter as Wink closes in on the light

     
     if(speedBoost > 180){             // uncomment this section to make Wink "scream" when light is above a certain level
        eyesRed(speedBoost);
        digitalWrite(Beeper,HIGH);
     }
     else {
        eyesPrevCol();
        digitalWrite(Beeper,LOW);
     }


     if(sensorDiff >= deadband){                              // if the difference is greater than the allowed deadband... then begin a turning action
        if(rightSensor>leftSensor){                                                //if the Right sensor was a larger value, begin a right turn
         motors((leftMotorSpeed+turnStrength),(rightMotorSpeed-turnStrength));     //Add POSITIVE turn strength to left, and NEGATIVE turn strength to right = right turn
        }
        else{                                                                      //if the Right sensor was not greater, then the Left must have been. Turn left.
         motors((leftMotorSpeed-turnStrength),(rightMotorSpeed+turnStrength));     //Add NEGATIVE turn strength to left, and POSITIVE turn strength to right = left turn
        }
     }
     else{
      motors(leftMotorSpeed,rightMotorSpeed);                                      //if light difference is within the dead band (both sensors picking up about the same
     }
     delay(5);

     if(buttonPressed()){                      //Exit this behavior if button is pressed
        motors(0,0);                            //make sure motors are stopped
        for(i=0;i<2;i++){
          eyesRGB(0,120,0);
          beep(20);
          delay(10);
          eyesRGB(0,0,0);
          delay(100);
        }
        delay(300);
        break;  //break causes program to exit the while(1) loop
    }
     
  }//end of while(1);
}
////////////////////////////////////////////
/// End Of Light Seeking Behavior Code   ///
////////////////////////////////////////////



////////////////////////////////////////////
/// IR Remote Behavior Code   //////////
////////////////////////////////////////////
void winkIRBehavior(void){

byte button;
 int i;
 
  RxIRRestart(4);         //wait for 4 byte IR remote command
while(1){


  if(IsIRDone()){             // if an IR packet has been received...
    button = GetIRButton();   // read which button was pressed
    Serial.println(button);

    switch(button){

        case 11:            // FWD Key Pressed
        motors(165,150);
        RxIRRestart(4);
        delay(100);
        motors(0,0);
        break;

        case 14:            // BACK Key Pressed
        motors(-150,-150);
        RxIRRestart(4);
        delay(100);
        motors(0,0);
        break;

        case 12:            // LEFT Key Pressed
        motors(-150,150);
        RxIRRestart(4);
        delay(60);
        motors(0,0);
        break;

        case 13:            // RIGHT Key Pressed
        motors(150,-150);
        RxIRRestart(4);
        delay(60);
        motors(0,0);
        break;

        case 18:            // "A" Key Pressed
        motors(-255,255);
        RxIRRestart(4);
        delay(1000);
        motors(0,0);
        break;        

        case 19:            // "B" Key Pressed
        motors(255,-255);
        RxIRRestart(4);
        delay(1000);
        motors(0,0);
        break;    

        case 21:            // "X" Key Pressed
        beep(50);
        RxIRRestart(4);
        break;
        
        case 1:             // 1 Key Pressed
        eyesRed(100);
        RxIRRestart(4);
        break;

        case 2:             // 2 Key Pressed
        eyesGreen(100);
        RxIRRestart(4);
        break;

        case 3:             // 3 Key Pressed
        eyesBlue(100);
        RxIRRestart(4);
        break;
        
        case 4:             // 4 Key Pressed
        eyesPurple(100);
        RxIRRestart(4);
        break;
        
        case 5:             // 5 Key Pressed
        eyesPink(100);
        RxIRRestart(4);
        break;

        case 6:             // 6 Key Pressed
        eyesCyan(100);
        RxIRRestart(4);
        break;
        
        case 7:             // 7 Key Pressed
        eyesOrange(100);
        RxIRRestart(4);
        break;
        
        case 8:             // 8 Key Pressed
        eyesYellow(100);
        RxIRRestart(4);
        break;

        case 9:             // 9 Key Pressed
        eyesWhite(100);
        RxIRRestart(4);
        break;

        case 10:            // 10 Key Pressed
        eyesOff();
        RxIRRestart(4);
        break;
        
    }
    
  } // end of if(IsIRDone())

if(buttonPressed()){                  //Exit this behavior if button is pressed
      motors(0,0);                        //make sure motors are stopped
      for(i=0;i<2;i++){
        eyesRGB(0,120,0);
        beep(20);
        delay(10);
        eyesRGB(0,0,0);
        delay(100);
      }
      delay(300);
      break;  //break causes program to exit the while(1) loop
    }

} //closing curly of the “while()” function
  
}
////////////////////////////////////////////
/// End Of IR Remote Behavior Code   ///
////////////////////////////////////////////



// Barrier Detect Code - This is the code that actually measures for a barrier
// it is a good example for writing your own light sensing code
int barrierDetect(void){

  int leftOnLevel,centerOnLevel,rightOnLevel;
  int leftOffLevel,centerOffLevel,rightOffLevel;
  
  digitalWrite(HeadLight,HIGH);               //turn on the IR headlight
  delayMicroseconds(200);                     //wait 200uS for sensors to stabalize before reading
  leftOnLevel = analogRead(AmbientSenseLeft);      //read all 3 ambient sensors with headlight turned on
  centerOnLevel = analogRead(AmbientSenseCenter);
  rightOnLevel = analogRead(AmbientSenseRight);
  
  digitalWrite(HeadLight,LOW);                //turn off the IR headlight
  delayMicroseconds(200);                     //wait 200uS for sensors to stabalize before reading
  leftOffLevel = analogRead(AmbientSenseLeft);     //read all 3 ambient sensors with headlight turn off
  centerOffLevel = analogRead(AmbientSenseCenter);
  rightOffLevel = analogRead(AmbientSenseRight);

  leftReflection = leftOnLevel-leftOffLevel;                  //subtact the "off" value so we're left with just the
  centerReflection = centerOnLevel-centerOffLevel;            //value from reflected light coming back into the sensor
  rightReflection = rightOnLevel-rightOffLevel;
  
  if (centerReflection < 0){                        //it's possible this value is negative, so we'll make sure
    centerReflection=0;                             //it is no lower than zero for all sensors
  }
  if (leftReflection < 0){                          //you can use the value centerReflection, leftReflection, and rightReflection
    leftReflection=0;                               //anywhere in your program after calling barrierDetect
  }
  if (rightReflection < 0){
    rightReflection=0;
  }
  return centerReflection;                          //return the value for centerReflection
}





